using System;
using System.Collections;
using VaultClientOperationsLib;
using VaultLib;
using VaultClientNetLib.ClientService; 
using System.Resources;
using System.Reflection;

namespace HistoryQuery
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Class1
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			string hostname, username, password, repository;
			
			hostname = "localhost";
			username = "admin";
			password = "adminpw";
			repository = "Default Repository";
			
			if (hostname.StartsWith("http://") == false)
				hostname = "http://" + hostname;
			if (hostname.EndsWith("/VaultService") == false)
				hostname += "/VaultService";
			ClientInstance myClient = new ClientInstance();
			myClient.Init(VaultClientNetLib.VaultConnection.AccessLevelType.Client);
			myClient.Login(hostname, username, password);
			FindRepository(myClient, repository);

			//Load the history for the past 1 day for $ and below.
			LoadHistory(myClient, "$", DateTime.Now.AddDays(-(double)1));
			myClient.Logout();
		}

		public static void LoadHistory(ClientInstance ciUser, string repositoryPath, DateTime dtBegin)
		{
			
			VaultLib.VaultHistoryQueryRequest req = new VaultLib.VaultHistoryQueryRequest();
			DateTime now = DateTime.Now;
			ArrayList users = new ArrayList();
			ArrayList substrings = new ArrayList();
			ArrayList actions = new ArrayList();
			ArrayList sorts = new ArrayList();
			sorts.Add((long)VaultQueryRequestSort.DateSort);
			req.SubstringType = VaultLib.VaultQueryRequestSubstrings.DoNotFilter;
			req.IsFolder = true;
			req.Recursive = true;
			
			req.DateFilterType = VaultQueryRequestDateTypes.IncludePastDays;
			req.DateFilterMask = VaultQueryRequestDates.HistoryBefore | VaultQueryRequestDates.HistoryAfter;
			req.EndDate = DateTime.Now;
			req.BeginDate = dtBegin;
			actions.Clear();
			req.CommentFilter = VaultLib.VaultQueryRequestComments.DoNotFilter;
			
			req.RepID = ciUser.ActiveRepositoryID;
			req.TopName = repositoryPath;

			req.Recursive = true;
			req.Users = (VaultLib.VaultUser[]) users.ToArray(typeof(VaultLib.VaultUser));
			req.Substrings = (string[]) substrings.ToArray(typeof(string));
			req.Actions = (long[]) actions.ToArray(typeof(long));
			req.Sorts = (long[]) sorts.ToArray(typeof(long));

			string token = String.Empty;
			int rowsRetrieved = 0;
			VaultLib.VaultHistoryItem[] items = new VaultLib.VaultHistoryItem[1];
			ciUser.Connection.HistoryBegin(req, 100, ref rowsRetrieved, ref token);
			if (rowsRetrieved > 0)
			{
				ciUser.Connection.HistoryFetch(token, 0, rowsRetrieved - 1, ref items);
			}
			ciUser.Connection.HistoryEnd(token);
			
			string strFormat = "On {0}, User {1} performed the action \"{2}\" on {3} with comment \"{4}\"";
			if (rowsRetrieved > 0)
			{
				ResourceManager rm = new ResourceManager("VaultClientPresentationLib.Resources.HistoryExplorerStrings", Assembly.Load("VaultPresentationLib"));
				foreach (VaultLib.VaultHistoryItem item in items)
				{
					string strAction;
					#region translate histitemtype to a string that makes sense.
					switch (item.HistItemType)
					{
						case VaultHistoryType.Added:
							strAction = String.Format("{0} {1}", rm.GetString("ActionAdded"), item.MiscInfo1);
							break;
						case VaultHistoryType.BranchedFrom:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionBranchedItem"), item.MiscInfo2, rm.GetString("From"), item.MiscInfo1);
							break;
						case VaultHistoryType.BranchedFromShare:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionBranchedItem"), item.MiscInfo1, rm.GetString("From"), rm.GetString("Share"));
							break;
						case VaultHistoryType.BranchedFromItem:
							strAction = String.Format("{0} {1} {2}", rm.GetString("ActionBranchedItem"), rm.GetString("From"), item.MiscInfo1);
							break;
						case VaultHistoryType.BranchedFromShareItem:
							strAction = String.Format("{0} {1} {2}", rm.GetString("ActionBranchedItem"), rm.GetString("From"), rm.GetString("Share"));
							break;
						case VaultHistoryType.CheckIn:
							strAction = rm.GetString("ActionCheckIn");
							break;
						case VaultHistoryType.Created:
							strAction = rm.GetString("ActionCreated");
							break;
						case VaultHistoryType.Deleted:
							strAction = String.Format("{0} {1}", rm.GetString("ActionDeleted"), item.MiscInfo1);
							break;
						case VaultHistoryType.Label:
							strAction = String.Format("{0} {1} ", rm.GetString("ActionLabeled"), item.MiscInfo1);
							break;
						case VaultHistoryType.MovedFrom:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionMovedFrom"), item.MiscInfo1, rm.GetString("From"), item.MiscInfo2);
							break;
						case VaultHistoryType.MovedTo:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionMovedTo"), item.MiscInfo1, rm.GetString("To"), item.MiscInfo2);
							break;
						case VaultHistoryType.Obliterated:
							strAction = String.Format("{0} {1}", rm.GetString("ActionObliterated"), item.MiscInfo1);
							break;
						case VaultHistoryType.Pinned:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionPinned"), item.MiscInfo1, rm.GetString("AtVersion"), item.MiscInfo2);
							break;
						case VaultHistoryType.PropertyChange:
							//we get this in the form E:x,M:y where E: marks the change for eol id, and M: marks the change for mergeable. 
							//x will be the new value of the eolid and y will be the new value of the mergeable setting.
							//x or y of 0 means no change and -1 means the value wasn't exp
							try
							{
								string s = item.MiscInfo1;
								string[] props = item.MiscInfo1.Split(',');
								string propName = string.Empty;
								string propValue = string.Empty;
								string[] propNames = null;
								string[] propValues = null;
								strAction = "";

								if (props != null)
								{
									propNames = new string[props.Length];
									propValues = new string[props.Length];
									for (int j=0; j<props.Length; j++)
									{
										string[] tmp = props[j].Split(':');
										if (int.Parse(tmp[1]) == -100)
										{
											propNames[j] = null;
											propValues[j] = null;
										}
										else 
										{
											if (tmp[0] == "E")
											{
												propNames[j] = rm.GetString("EOLConversion");
												switch (int.Parse(tmp[1]))
												{
													case VaultLib.VaultEOLForm.CR:
														propValues[j] = rm.GetString("CR");
														break;
													case VaultLib.VaultEOLForm.CRLF:
														propValues[j] = rm.GetString("CRLF");
														break;
													case VaultLib.VaultEOLForm.LF:
														propValues[j] = rm.GetString("LF");
														break;
													case VaultLib.VaultEOLForm.Native:
														propValues[j] = rm.GetString("Native");
														break;
													case VaultLib.VaultEOLForm.None:
														propValues[j] = rm.GetString("None");
														break;
												}										
											}

											else if (tmp[0] == "M")
											{
												propNames[j] = rm.GetString("FileType");
												switch (int.Parse(tmp[1]))
												{
													case VaultLib.VaultMergeable.Binary:
														propValues[j] = rm.GetString("Binary");
														break;
													case VaultLib.VaultMergeable.Mergeable:
														propValues[j] = rm.GetString("Mergeable");
														break;
													case VaultLib.VaultMergeable.UseSystemDefault:
														propValues[j] = rm.GetString("UseSystemDefault");
														break;														
												}										
											}								
										}
									}						
								}
							
								for (int n = 0; n < propNames.Length; n++)
								{
									if (propNames[n] != null)
									{
										if (strAction.Length > 0)
											strAction += ", ";

										strAction += string.Format("{0} {1} {2}",  propNames[n], rm.GetString("changedTo"), propValues[n]);
									}
								}
							}
							catch
							{
								strAction = rm.GetString("PropertyChanged");
							}
					
							break;
						case VaultHistoryType.Renamed:
							strAction = String.Format("{0} {1} {2} {3} {4}", rm.GetString("ActionRenamedItem"), rm.GetString("From"), item.MiscInfo2, rm.GetString("To"), item.MiscInfo1);
							break;
						case VaultHistoryType.RenamedItem:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionRenamed"), item.MiscInfo2, rm.GetString("To"), item.MiscInfo1);
							break;
						case VaultHistoryType.Rollback:
							//strAction = rm.GetString("ActionRollback");
							strAction = String.Format("{0} {1} {2}", rm.GetString("ActionRollback"), rm.GetString("ToVersion"), item.MiscInfo1);
							break;
						case VaultHistoryType.SharedTo:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionShared"), item.MiscInfo1, rm.GetString("As"), item.MiscInfo2);
							break;
						case VaultHistoryType.Undeleted:
							strAction = String.Format("{0} {1}", rm.GetString("ActionUndeleted"), item.MiscInfo1);
							break;
						case VaultHistoryType.UnPinned:
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionUnPinned"), item.MiscInfo1, rm.GetString("FromVersion"), item.MiscInfo2);
							break;
						case VaultHistoryType.Snapshot:
							strAction = String.Format("{0} {1} ", rm.GetString("ActionSnapshot"), item.MiscInfo1);
							break;
						case VaultHistoryType.SnapshotFrom:
							string snapshotName = string.Empty;
							//display only the name of the labeled folder, not the full path
							if (item.MiscInfo2 != null)
							{
								string[] pathItems = item.MiscInfo2.Split('/');
								snapshotName = pathItems[pathItems.Length-1];
							}
							strAction = String.Format("{0} {1} {2} {3}", rm.GetString("ActionSnapshot"), item.MiscInfo1, rm.GetString("As"), snapshotName);
							break;
						case VaultHistoryType.SnapshotItem:
							strAction = String.Format("{0} {1} {2} ", rm.GetString("ActionSnapshot"), rm.GetString("From"), item.MiscInfo1);
							break;
						default:
							strAction = rm.GetString("ActionUnknown");
							break;
					}
					#endregion
					Console.WriteLine(strFormat, item.TxDate, item.UserLogin, strAction, item.Name, item.Comment);
				}

				
			}
		}
		/// <summary>
		/// This is a helper function that works around a deficiency in the 
		/// Vault Client API.  In the API, there is no method that lets you set the active repository
		/// for a ClientInstance based only on the repository name.  This means that (for now) every
		/// program that uses the API will need to do something like this.
		/// </summary>
		/// <param name="client">A Client Instance that has already had Init and Login called on it.</param>
		/// <param name="repositoryName">The name of the repository to find and connect to.</param>
		public static void FindRepository(ClientInstance client, string repositoryName)
		{
			VaultRepositoryInfo[] reps = null;
			//List all the repositories on the server.
			client.ListRepositories(ref reps);

			//Search for the one that we want.
			foreach (VaultRepositoryInfo r in reps)
			{
				if (String.Compare(r.RepName,repositoryName, true) == 0)
				{
					//This will load up the client side cache files and refresh the repository structure.
					//See http://support.sourcegear.com/viewtopic.php?t=6 for more on client side cache files.
					client.SetActiveRepositoryID(r.RepID, client.Connection.Username, r.UniqueRepID, true, true);
					break;
				}
			}
			if (client.ActiveRepositoryID == -1)
				throw new Exception(string.Format("Repository {0} not found", repositoryName));
		}

	}
}
